Posit

Struct Posit 

pub struct Posit<const N: u32, const ES: u32, Int: Int>(/* private fields */);
A posit floating point number with N bits and ES exponent bits, using Int as its underlying type.

Examples:

type Foo = Posit::<32, 2, i32>;  // A 32-bit posit with 2-bit exponent field, represented in a
                                 // 32-bit machine type
type Bar = Posit::<6, 1, i8>;  // A 6-bit posit with 1-bit exponent field, represented in an
                               // 8-bit machine type.

Type aliases are provided at the crate root for the posit types defined in the standard.

Implementations

Source

impl<const N: u32, const ES: u32, Int: Int> Posit<N, ES, Int>

pub const BITS: u32

The size of this Posit type in bits.

Note: this is the logical size, not necessarily the size of the underlying type.

pub const ES: u32

The number of exponent bits.

pub fn from_bits(x: Int) -> Self

Construct a posit from its raw bit representation. Bits higher (more significant) than the lowest N (Self::BITS) bits, if any, are ignored.

pub const unsafe fn from_bits_unchecked(x: Int) -> Self

As Self::from_bits, but if x is not a result of a Self::to_bits call, then calling this function is undefined behaviour.

pub const fn to_bits(self) -> Int

Return the underlying bit representation of self as a machine int. Bits higher (more significant) than the lowest N (Self::BITS) bits, if any, are set as equal to the N-1th bit (i.e. sign-extended).

Source

impl<const N: u32, const ES: u32, Int: Int> Posit<N, ES, Int>

pub const ZERO: Self

Zero (0), the additive identity element.

pub const NAR: Self

Not-a-real (NaR).

pub const ONE: Self

One (1), the additive identity element.

pub const MINUS_ONE: Self

Negative one (-1).

pub const MAX: Self

Largest representable value, equal to -MIN.

pub const MIN: Self

Smallest representable value, equal to -MAX.

Not to be confused with the smallest absolute value, i.e. Self::MIN_POSITIVE.

pub const MIN_POSITIVE: Self

Smallest positive value, equal to -MAX_NEGATIVE.

pub const MAX_NEGATIVE: Self

Largest negative value, equal to -MIN_POSITIVE.

pub const MIN_EXP: Int

The minimum exponent; Self::MIN_POSITIVE = 2 Self::MIN_EXP.

pub const MAX_EXP: Int

The maximum exponent; Self::MAX_NEGATIVE = 2 Self::MAX_EXP.

Source

impl<const N: u32, const ES: u32, Int: Int> Posit<N, ES, Int>

pub fn next(self) -> Self

Returns the posit value of the lexicographic successor of self’s representation.

Standard: “next”.

pub fn prior(self) -> Self

Returns the posit value of the lexicographic predecessor of self’s representation.

Standard: “prior”.